package ipc

import (
	"fmt"
	"net"

	"idio.link/go/ipc/common"
	"idio.link/go/ipc/protocols"
)

func NewClient() *Client {
	return &Client{
		cpm: protocols.NewClientPM(),
		log: common.NewLogger(),
	}
}

type Client struct {
	conn net.Conn
	cpm  *protocols.ClientPM
	log  *common.Logger
}

func (c *Client) Alloc() (err error) {
	c.log.Debug("connect to ipc service")

	abstract := "\000idio.link/go/ipc"
	c.conn, err = net.Dial("unixpacket", abstract)
	if err != nil {
		err = fmt.Errorf("ipc client alloc: %v", err)
		return
	}

	c.log.Debug("send allocation request")

	buf := make([]byte, 1024)
	sdu := c.cpm.ReqAlloc(buf)
	err = common.Send(c.conn, sdu)
	if err != nil {
		_ = c.Close()
		err = fmt.Errorf("send: %v", err)
		return
	}
	sdu, err = common.Receive(c.conn, buf)
	if err != nil {
		_ = c.Close()
		err = fmt.Errorf("receive: %v", err)
	}
	c.log.Debug("got %q\n", sdu)
	return
}

func (c *Client) Close() (err error) {
	err = c.conn.Close()
	if err != nil {
		err = fmt.Errorf("ipc client: failed to close connection: %v", err)
	}
	return
}
